package edu.neu.ccs.buyer;

import java.io.File;
import java.io.FileInputStream;

import junit.framework.TestCase;
import edu.neu.ccs.demeterf.demfgen.lib.List;
import gen.Assignment;
import gen.Derivative;
import gen.Literal;
import gen.Pos;
import gen.RawMaterial;

/**
 * Test class for Buyer
 * @author Charu Chandra
 *
 */
public class BuyerTest extends TestCase {
	
	public void testPrice(){
		try{
			Buyer buyer = new Buyer(6,"testBuyer1");
			//Test1
			Derivative d1 = Derivative.parse("deriv[\"d1\" 6  0.504444111 type[22]]");
			assertEquals(buyer.goodBuy(d1), false);
			assertEquals(buyer.goodBuy(d1,0.6), false);
			
			//Test2
			Derivative d2 = Derivative.parse("deriv[\"d2\" 6  0.43 type[22]]");
			assertEquals(buyer.goodBuy(d2), true);
			assertEquals(buyer.goodBuy(d2,0.1), false);
			assertEquals(buyer.goodBuy(d2,0.01), true);
			
			//Test3
			Derivative d3 = Derivative.parse("deriv[\"d3\" 6  0.618 type[170 119]]");
			assertEquals(buyer.goodBuy(d3), true);
			assertEquals(buyer.goodBuy(d3,0.01), false);
			assertEquals(buyer.goodBuy(d3,0.001), false);
			
			//Test4
			Derivative d4 = Derivative.parse("deriv[\"d4\" 6  0.666 type[170 119]]");
			assertEquals(buyer.goodBuy(d4), false);
			assertEquals(buyer.goodBuy(d4,0.2), false);
			assertEquals(buyer.goodBuy(d4,0.0001), false);
			
			//Test5
			Derivative d5 = Derivative.parse("deriv[\"d5\" 6  0.139 type[2 6]]");
			assertEquals(buyer.goodBuy(d5), true);
			assertEquals(buyer.goodBuy(d5,0.2), false);
			assertEquals(buyer.goodBuy(d5,0.001), true);
			
			//Test6
			Derivative d6 = Derivative.parse("deriv[\"d6\" 6  0.208 type[2 6]]");
			assertEquals(buyer.goodBuy(d6), false);
			assertEquals(buyer.goodBuy(d6,0.2), false);
			assertEquals(buyer.goodBuy(d6,0.001), false);
			
			//Test7
			Derivative d7 = Derivative.parse("deriv[\"d5\" 6  0.1 type[6]]");
			assertEquals(buyer.goodBuy(d7), true);
			assertEquals(buyer.goodBuy(d7,0.1), true);
			assertEquals(buyer.goodBuy(d7,0.2), false);
			
			//Test8
			Derivative d8 = Derivative.parse("deriv[\"d6\" 6  0.30 type[6]]");
			assertEquals(buyer.goodBuy(d8), false);
			assertEquals(buyer.goodBuy(d8,0.1), false);
			assertEquals(buyer.goodBuy(d8,0.000001), false);
			
		}catch (Exception e) {
			System.out.println("Exception in tests: " + e);
		}
	}
	
	/**
	 * Naming convention of files used rm_listOfRelationNos_noOfVars
	 */
	public void testAssignment(){
		try {
			
			Buyer buyer = new Buyer(6,"testBuyer2");
			
			//Test1
			//-----
			
			//Testing assignment
			File rmFile1 = new File("./testdata/rm_170_119_10");
			FileInputStream rmIs1 = new FileInputStream(rmFile1);
			RawMaterial rm1 = RawMaterial.parse(rmIs1);
			// The result reaches the maximum when 8 out of 10 are set to true
			//but since we have two dont cares, the assignment gives 4
			assertEquals(countOnes(buyer.getFinishedProduct(rm1)),4);
			
			//Testing quality
			Assignment assign11 = Assignment.parse("[  X0    dc2    dc3    X1   ! X2   ! X3   ! X4   ! X5   ! X6   ! X7   ! X8   ! X9  ]");
			assertEquals(Buyer.findQuality(rm1, assign11),0.81,0.01);
			
			Assignment assign12 = Assignment.parse("[  X0    ! dc2    ! dc3    X1   ! X2   ! X3   ! X4   ! X5   ! X6   ! X7   ! X8   ! X9  ]");
			assertEquals(Buyer.findQuality(rm1, assign12),0.81,0.01);
			
			Assignment assign13 = Assignment.parse("[  X0    dc2    dc3    X1    X2   X3   ! X4   ! X5   ! X6   ! X7   ! X8   ! X9  ]");
			assertEquals(Buyer.findQuality(rm1, assign13),0.76,0.01);
			
			//Test2
			//-----
			
			//Testing assignment
			File rmFile2 = new File("./testdata/rm_2_9");
			FileInputStream rmIs2 = new FileInputStream(rmFile2);
			RawMaterial rm2 = RawMaterial.parse(rmIs2);  
			assertEquals(countOnes(buyer.getFinishedProduct(rm2)),3);
			
			//Testing quality
			Assignment assign21 = Assignment.parse("[X0    X1   ! X2   ! X3   ! X4   ! X5   ! X6   ! X7    X8]");
			assertEquals(Buyer.findQuality(rm2, assign21),0.18,0.01);
			
			Assignment assign22 = Assignment.parse("[X0    X1   ! X2   ! X3   ! X4   ! X5   X6   X7    X8]");
			assertEquals(Buyer.findQuality(rm2, assign22),0.11,0.01);
			
			Assignment assign23 = Assignment.parse("[!X0  !X1   ! X2   ! X3   ! X4   ! X5   ! X6   ! X7    !X8]");
			assertEquals(Buyer.findQuality(rm2, assign23),0.0,0.0);
			
			//Test3
			//-----
			
			//Testing assignment
			File rmFile3 = new File("./testdata/rm_24_155_9");
			FileInputStream rmIs3 = new FileInputStream(rmFile3);
			RawMaterial rm3 = RawMaterial.parse(rmIs3);  
			assertEquals(countOnes(buyer.getFinishedProduct(rm3)),4);
			
			
			//Test4
			//-----
			
			//Testing assignment
			File rmFile4 = new File("./testdata/rm_22_4");
			FileInputStream rmIs4 = new FileInputStream(rmFile4);
			RawMaterial rm4 = RawMaterial.parse(rmIs4);  
			assertEquals(countOnes(buyer.getFinishedProduct(rm4)),1);
			
			//Testing quality
			Assignment assign41 = Assignment.parse("[X0    ! X1   ! X2   ! X3]");
			assertEquals(Buyer.findQuality(rm4, assign41),0.75,0.01);
			
			Assignment assign42 = Assignment.parse("[X0    X1   ! X2   ! X3]");
			assertEquals(Buyer.findQuality(rm4, assign42),0.5,0.01);
			
			Assignment assign43 = Assignment.parse("[X0    X1   X2   ! X3]");
			assertEquals(Buyer.findQuality(rm4, assign43),0.0,0.0);
			
			//Test5
			//-----
			File rmFile5 = new File("./testdata/rm_170_119_20");
			FileInputStream rmIs5 = new FileInputStream(rmFile5);
			RawMaterial rm5 = RawMaterial.parse(rmIs5);
			// The result reaches the maximum when 18 out of 20 are set to true
			//but since we have two dont cares, the assignment gives 4
			assertEquals(countOnes(buyer.getFinishedProduct(rm5)),4);
			
		} catch (gen.ParseException e) {
			System.err.println("Could not parse raw material");
		} catch (Exception e){
			System.err.println("Exception in tests: " + e);
		}
	}
	
	private int countOnes(Assignment assign){
		List<Literal> literals = assign.literals;
		int count = 0;
		
		for(Literal lit: literals){
			if(lit.value instanceof Pos) count++;
		}
		
		return count;
	}
}
