package edu.neu.ccs.seller;

import java.io.File;
import java.io.FileInputStream;

import edu.neu.ccs.buyer.Buyer;
import gen.Derivative;
import gen.RawMaterial;
import gen.Type;
import junit.framework.TestCase;

/**
 * Test class for Seller
 * @author Charu Chandra
 *
 */
public class SellerTest extends TestCase {
	
	public void testRawMaterial(){
		Seller seller = new Seller(6, "testSeller1");
		try {
			
			//Test1
			//symmetric formula with dont cares
			RawMaterial rm1 = seller.generateRawMaterial(10, Type.parse("type[170 119]"));
			assertEquals(countNoOfConstraints(rm1),55);
			
			//Test2
			//increasing number of variables
			RawMaterial rm2 = seller.generateRawMaterial(20, Type.parse("type[170 119]"));
			assertEquals(countNoOfConstraints(rm2),210);
			
			//Test3
			//increasing number of variables
			RawMaterial rm3 = seller.generateRawMaterial(30, Type.parse("type[170 119]"));
			assertEquals(countNoOfConstraints(rm3),465);
			
			//Test4
			//symmetrical formula
			RawMaterial rm4 = seller.generateRawMaterial(10, Type.parse("type[22]"));
			assertEquals(countNoOfConstraints(rm4),120);
			
			//Test4
			//asymmetric formula
			RawMaterial rm5 = seller.generateRawMaterial(10, Type.parse("type[2]"));
			assertEquals(countNoOfConstraints(rm5),720);
		
		
		} catch (Exception e) {
			System.err.println("Exception in test: " + e);
		}
		
	}
	
	/**
	 * Test generated derivatives if they are rightly priced or not
	 */
	public void testDerivative(){
		Seller seller = new Seller(6, "testSeller2");
		Buyer buyer = new Buyer(7, "testBuyer3");
		
		try {
			 
			//Test1
			Derivative d1 = seller.generateDerivative("d1");
			System.out.println("Generated derivative 1: " + d1.print());
			assertEquals(false, buyer.goodBuy(d1));
			
			//Test2
			Derivative d2 = seller.generateDerivative("d2");
			System.out.println("Generated derivative 2: " + d2.print());
			assertEquals(false, buyer.goodBuy(d2));
			
			//Test3
			Derivative d3 = seller.generateDerivative("d3");
			System.out.println("Generated derivative 2: " + d3.print());
			assertEquals(false, buyer.goodBuy(d3));
			
		} catch (Exception e) {
			System.err.println("Exception in test: " + e);
		}
		
	}
		
	private int countNoOfConstraints(RawMaterial rm){
		return rm.instance.cs.length();
	}
	
}
