package edu.neu.ccs.bias;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import edu.neu.ccs.demeterf.demfgen.lib.Empty;
import edu.neu.ccs.demeterf.demfgen.lib.List;
import edu.neu.ccs.satsolver.PolynomialI;

/**
 * Class for calculating max bias and price.
 * <p>
 * Price and bias for all 255 relations are pre-calculated and stored in maps
 * which are read whenever required. This way we save on calculating price
 * and bias each time a relation is used. 
 * 
 * @author Charu Chandra and Wu Jiang
 *
 */
public class BiasCalculator{
    
    private static BiasCalculator instance = null;
    
    //Creating maps for storing price and bias values
    private static java.util.Map<Integer,Double> priceMap = new HashMap<Integer,Double>();
    private static java.util.Map<Integer,Double> biasMap = new HashMap<Integer,Double>();
    
    public static void p(String s) {
	System.out.println(s);
    }
    
    private BiasCalculator(){
	buildPriceList();
    }
    
    /**
     * Get an instance of BiasCalculator class
     * @return
     */
    public static BiasCalculator getInstance(){
	if (instance == null) {
	    instance = new BiasCalculator();
	}
		
	return instance;
    }

    private void buildPriceList(){
	List<Integer> relationList = new Empty<Integer>();
	relationList = relationList.buildlist(new List.Build<Integer>() {
	    public Integer build(int i) {
		return new Integer(i);
	    }
	}, 256);
	
	
	for(Integer r: relationList){
	    Set<Pair> set = new HashSet<Pair>();
	    Pair pair = new Pair(r, 1);
	    set.add(pair);
	    InputInitial input = new InputInitial(set);
	    double maxBias = (ClassicSATSolverUtil.calculateBias(input))
	    .getMaxBias();
	    PolynomialI poly = (ClassicSATSolverUtil.calculateBias(input))
	    .getPolynomial();
	    priceMap.put(r, new Double(poly.eval(maxBias)));
	    biasMap.put(r, new Double(maxBias));
	}
    }
    
    /**
     * Returns price for the input relation number. 
     * The price is calculated by maximizing probability equation wrt bias. 
     * @param relation
     * @return
     */
    public double getPrice(int relation) {
	return priceMap.get(new Integer(relation)).doubleValue();
    }

    /**
     * Calculate bias for the input relation number
     * The maxBias is the value of bias that maximizes the probability function.
     * @param relation
     * @return
     */
    public double getBias(int relation) {
	return biasMap.get(new Integer(relation)).doubleValue();
    }
    
    /**
     * Gets the bias function for the relation
     * @param relation
     * @return
     */
    public PolynomialI getPolynomial(int relation){
    	Pair pair = new Pair(new Integer(relation),1);
    	Set set = new HashSet();
    	set.add(pair);
    	return ClassicSATSolverUtil.calculateBias(new InputInitial(set)).getPolynomial();
    }

}