package edu.neu.ccs.buyer;

import edu.neu.ccs.assignment.AssignmentUtil;
import edu.neu.ccs.demeterf.demfgen.lib.List;
import edu.neu.ccs.util.MinMaxSolver;
import gen.Assignment;
import gen.Derivative;
import gen.Player;
import gen.PlayerID;
import gen.Price;
import gen.RawMaterial;
import gen.Type;
import gen.TypeInstance;

/**
 * Represents a buyer. Provides all teh functions that a buyer might want to do while
 * playing SDG game
 * 
 * @author Charu Chandra and Wu Jiang
 *
 */
public class Buyer extends Player{

	public Buyer(int id, String name){
		super(new PlayerID(id), name);
	}
	/**
	 * Suggests if the derivative with a list of relations is a good buy or not
	 * Takes only two relations
	 * @param Derivative
	 * @param price
	 * @return boolean 
	 */
	public boolean goodBuy(Derivative d){
		return goodBuy(d.type,d.price);
	}

	/**
	 * Suggests if the derivative with a list of relations is a good buy or not.
	 * The derivative should make at least p amount of profit. Takes only two relations
	 * @param Derivative
	 * @param price
	 * @return boolean 
	 */
	public boolean goodBuy(Derivative d, double p){
		return goodBuy(d.type,new Price(d.price.val + p));
	}

	/**
	 * Suggests if the derivative with a list of relations is a good buy or not
	 * @param type
	 * @param price
	 * @return boolean 
	 */
	public boolean goodBuy(Type type, Price price){
		List<TypeInstance> relations = type.instances;
		return (MinMaxSolver.getOptimumPrice(relations) 
				>= price.val);
	}

	/**
	 * Provides a finished product for the given raw material
	 * @param rm
	 * @return
	 * @throws Exception
	 */
	public Assignment getFinishedProduct(RawMaterial rm) throws Exception{
		return AssignmentUtil.findAssignment(rm);
	}

	/**
	 * Calculates the quality of assignment for the input raw material.
	 * 
	 * @param rawMaterial
	 *            in form of RawMaterial object
	 * @param assign
	 *            in form of Assignment object
	 * @return quality achieved by the assignment
	 * @throws Exception
	 *             Any exception produced during calculation
	 */
	public static double findQuality(RawMaterial rawMaterial, Assignment assign)
	throws Exception {
		return AssignmentUtil.findQuality(rawMaterial, assign);
	}

}
