package edu.neu.ccs.rawmaterial;

import edu.neu.ccs.demeterf.demfgen.lib.List;
import edu.neu.ccs.util.MinMaxSolver;

import gen.RawMaterial;
import gen.RelationNr;
import gen.Type;
import gen.TypeInstance;
import gen.Weight;

/**
 * Utility class for generating Raw material
 * @author Charu Chandra and Wu Jiang
 *
 */
public class RawMaterialUtil {

	/**
	 * Generates Raw material. It can currently generate raw material 
	 * for only for the first two relations.
	 * @param numVar - number of variables
	 * @param type - Type containing TypeInstance
	 * @return RawMaterial 
	 */
	public static RawMaterial generateRawMaterial(int numVar, Type type) throws Exception{
		List<TypeInstance> instances = type.instances;
		String rawMat = generateRawMaterial(numVar, instances);
		return RawMaterial.parse(rawMat);
	}
	/**
	 * Generates Raw material. It can currently generate raw material 
	 * for only two relations 
	 * @param numVar number of variables
	 * @param relation1 1st relation numbers
	 * @param relation2 2nd relation number
	 * @return
	 */
	public static String generateRawMaterial(int numVar,List<TypeInstance> relations) {
		if(relations.length() == 0 ) return "";
		
		int relation1;
		int relation2;
		
		StringBuffer rawMat = new StringBuffer("rm[ ");
		
		String[][] rawMaterialFormula1 = null;
		if(relations.length() == 1 ) {
			relation1 = relations.top().r.v;
			rawMaterialFormula1 = SymFormula.generateSymFormula(numVar, relation1);
			for (String[] row : rawMaterialFormula1) {
				rawMat.append( "1 : " + relation1 + " "); 
				for (String v : row) {
					rawMat.append(v + " ");
				}
			}	
			rawMat.append("]");
			return rawMat.toString();
		}
		
		relation1 = relations.top().r.v;
		relation2 = relations.pop().top().r.v;
		rawMaterialFormula1 = SymFormula.generateSymFormula(numVar, relation1);
		String[][] rawMaterialFormula2 = SymFormula.generateSymFormula(numVar, relation2);
		Weight[] weights = calculateWeight(new RelationNr(relation1), new RelationNr(relation2));
		for (String[] row : rawMaterialFormula1) {
			rawMat.append(weights[0].v + " : " + relation1 + " "); 
			for (String v : row) {
				rawMat.append(v + " ");
			}
		}

		for (String[] row : rawMaterialFormula2) {
			rawMat.append(weights[1].v + " : " + relation2 + " ");
			for (String v : row) {
				rawMat.append(v + " ");
			}
		}
		rawMat.append("]");
		return rawMat.toString();
	}

	/**
	 * Calculate optimum weights for two relations
	 * @param relation1
	 * @param relation2
	 * @return Weight array which has two values . First one is for relation1 and 
	 * 2nd for relation2
	 */
	private static Weight[] calculateWeight(RelationNr relation1, RelationNr relation2){
		double weightRatio = MinMaxSolver.getOptimumT(relation1.v, relation2.v);
		int x = (int)(weightRatio * 1000);
		int y = 1000 - x;
		Weight[] weights = {new Weight(x), new Weight(y)};
		return weights;
	}

}
