package edu.neu.ccs.util;
import edu.neu.ccs.demeterf.Bc;
import edu.neu.ccs.demeterf.Traversal;
import edu.neu.ccs.demeterf.demfgen.lib.List;
import edu.neu.ccs.evergreen.ir.Relation;
import gen.Constraint;
import gen.Literal;
import gen.Pos;
import gen.RawMaterial;
import gen.RelationNr;
import gen.Variable;
import gen.Weight;

/**
 * Performs operations related to shannon cofactor
 * @author Charu Chandra and Wu Jiang
 *
 */
public class ShannonCofactor extends Bc {

	Constraint combine(Constraint c, Weight w, RelationNr r,
			List<Variable> vars, Literal lit) {
		int newRelation = computeSCForConstraint(c, w, r, vars, lit);
		return new Constraint(w, new RelationNr(newRelation), vars);
	}

	private static RawMaterial computeShannonCofactor(RawMaterial s, Literal lit) {
		return new Traversal(new ShannonCofactor()).<RawMaterial> traverse(s,
				lit);
	}

	private static int computeSCForConstraint(Constraint c, Weight w, RelationNr r,
			List<Variable> vars, Literal lit) {
		Relation relation = new Relation(3, r.v);

		final Variable variable = (Variable) lit.var;
		final int value = (lit.value instanceof Pos) ? 1 : 0;
		
		int position = vars.index(new List.Pred<Variable>() {
			public boolean huh(Variable v) {
				return v.equals(variable);
			}
		});

		int newRelation = r.v;
		if (position != -1)
			newRelation = relation.reduce(position, value);

		return newRelation;
	}

	
	public static int computeSC(int r, int position, int value) {
		Relation relation = new Relation(3,r);
		return relation.reduce(position, value);
	}

}